@@ -32,6 +32,14 @@
 #' @param blockUnit Character string taking the values, "secs," "mins," 
 #'     "hours," "days," or "weeks." Describes the temporal unit associated with
 #'     each block. Defaults to "hours."
+#' @param blockingStartTime Character string or date object describing the date
+#'     OR dateTime starting point of the first time block. For example, if 
+#'     blockingStartTime = "2016-05-01" OR "2016-05-01 00:00:00", the first 
+#'     timeblock would begin at "2016-05-01 00:00:00." If NULL, the 
+#'     blockingStartTime defaults to the minimum dateTime point in x. Note: 
+#'     any blockingStartTime MUST precede or be equivalent to the minimum 
+#'     timepoint in x. Additional note: If blockingStartTime is a character 
+#'     string, it must be in the format ymd OR ymd hms.
 #' @param equidistant.time Logical. If TRUE, location fixes in individuals' 
 #'     movement paths are temporally equidistant (e.g., all fix intervals are 
 #'     30 seconds). Defaults to FALSE. Note: This is a time-saving argument. 
@@ -105,7 +113,7 @@
 #'     sec.threshold=10, blocking = FALSE, blockUnit = "hours", blockLength = 1,
 #'     equidistant.time = FALSE, parallel = FALSE, reportParameters = TRUE)
 
-contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE, blockLength = 1, blockUnit = "hours", equidistant.time = FALSE, parallel = FALSE, nCores = (parallel::detectCores()/2), reportParameters = TRUE){ 
+contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE, blockLength = 1, blockUnit = "hours", blockingStartTime = NULL, equidistant.time = FALSE, parallel = FALSE, nCores = (parallel::detectCores()/2), reportParameters = TRUE){ 
   
   #bind the following variables to the global environment so that the CRAN check doesn't flag them as potential problems
   l <- NULL
@@ -115,6 +123,8 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
     
     listBreak_dur.generator<-function(x, dist.threshold,sec.threshold, blocking, blockUnit, blockLength, equidistant.time, parallel, reportParameters, nCores){#this function just calls what happens when x is not a list o data frames
       
+      #write sub-functions here
+      
       durFinder.noblock<-function(parallel, dist.threshold, sec.threshold, equidistant.time, nCores, environmentTag){
         
         eval(expr = { #evaluate these function steps in the master-function environment to avoid cloning x (i.e., dist2... ouput), which can be quite large
@@ -179,8 +189,8 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
             
             return(durationTab)
           }
-          contactMatrix.maker <- function(x,idVec1, dist.all.reduced){
-            spec.dist = dist.all.reduced[which(dist.all.reduced$id == as.character(unname(unlist(x[1])))),]
+          contactMatrix.maker <- function(x,idVec1, x.reduced){
+            spec.dist = x.reduced[which(x.reduced$id == as.character(unname(unlist(x[1])))),]
             dt = spec.dist$dt
             timebreakVec <-which(dt > unname(unlist(x[3]))) #needed here. fixed 1/8
             distmat = data.matrix(spec.dist[,c(match(paste("dist.to.indiv_",as.character(idVec1),sep =""), names(spec.dist)))])
@@ -194,36 +204,35 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
             return(idDurations)
           }
           
-          dist.all<-x[order(x$id, x$dateTime),]
-          idVec1 = unique(dist.all$id)
+          idVec1 = unique(x$id)
           if(length(idVec1) > 1){
-            dist.all.reduced <-dist.all[-which(dist.all$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
+            x.reduced <-x[-which(x$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
             
             if(equidistant.time == TRUE){ #added 02/04/2019 to make the dt calculations a toggleable parameter that users may turn off if all data points in their data set are temporally equidistant. This saves a large amount of time (approx. 3.5 mins/day)
-              dist.all.reduced$dt = 0
+              x.reduced$dt = 0
             }else{ #if equidistant.time == FALSE
               
-              if(nrow(dist.all.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because dist.all.reduced is only created from blocks with observed contacts, there will never be a case where dist.all.reduced < 1)
-                dist.all.reduced$dt = 0
-              }else{ #if there's more than one row in dist.all.reduced
-                timesFrame = data.frame(dist.all.reduced$dateTime[1:(nrow(dist.all.reduced) - 1)], dist.all.reduced$dateTime[2:nrow(dist.all.reduced)], stringsAsFactors = TRUE)
+              if(nrow(x.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because x.reduced is only created from blocks with observed contacts, there will never be a case where x.reduced < 1)
+                x.reduced$dt = 0
+              }else{ #if there's more than one row in x.reduced
+                timesFrame = data.frame(x.reduced$dateTime[1:(nrow(x.reduced) - 1)], x.reduced$dateTime[2:nrow(x.reduced)], stringsAsFactors = TRUE)
                 if (parallel == TRUE){
                   cl<-parallel::makeCluster(nCores)
                   on.exit(parallel::stopCluster(cl))
                   timedif<-parallel::parApply(cl, timesFrame, 1, timeDifference)
-                  dist.all.reduced$dt = c(0, timedif) #timedif represents the time it takes to move from location i-1 to location i	
+                  x.reduced$dt = c(0, timedif) #timedif represents the time it takes to move from location i-1 to location i	
                 }else{
                   timedif = apply(timesFrame, 1, timeDifference) 
-                  dist.all.reduced$dt = c(0, timedif) 
+                  x.reduced$dt = c(0, timedif) 
                 }
               }
             }
-            comboFrame = data.frame(unique(dist.all.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
+            comboFrame = data.frame(unique(x.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
             
             if (parallel == TRUE){
-              duration<-parallel::parApply(cl, comboFrame, 1, contactMatrix.maker,idVec1, dist.all.reduced) #note that cl is defined above
+              duration<-parallel::parApply(cl, comboFrame, 1, contactMatrix.maker,idVec1, x.reduced) #note that cl is defined above
             }else{
-              duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, dist.all.reduced)	
+              duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, x.reduced)	
             }
             durationTable = data.frame(data.table::rbindlist(duration), stringsAsFactors = TRUE)
           }else{
@@ -296,8 +305,8 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
           
           return(durationTab)
         }
-        contactMatrix.maker <- function(x,idVec1, dist.all.reduced){
-          spec.dist = dist.all.reduced[which(dist.all.reduced$id == as.character(unname(unlist(x[1])))),]
+        contactMatrix.maker <- function(x,idVec1, x.reduced){
+          spec.dist = x.reduced[which(x.reduced$id == as.character(unname(unlist(x[1])))),]
           dt = spec.dist$dt
           timebreakVec <-which(dt > unname(unlist(x[3]))) #needed here. fixed 1/8
           distmat = data.matrix(spec.dist[,c(match(paste("dist.to.indiv_",as.character(idVec1),sep =""), names(spec.dist)))])
@@ -311,30 +320,29 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
           return(idDurations)
         }   
         
-        dist.all<-x[order(x$id, x$dateTime),]
-        idVec1 = unique(dist.all$id)
+        idVec1 = unique(x$id)
         if(length(idVec1) > 1){
-          dist.all.reduced <-dist.all[-which(dist.all$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
+          x.reduced <-x[-which(x$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
           
           if(equidistant.time == TRUE){ #added 02/04/2019 to make the dt calculations a toggleable parameter that users may turn off if all data points in their data set are temporally equidistant. This saves a large amount of time (approx. 3.5 mins/day)
-            dist.all.reduced$dt = 0
+            x.reduced$dt = 0
           }else{ #if equidistant.time == FALSE
-            if(nrow(dist.all.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because dist.all.reduced is only created from blocks with observed contacts, there will never be a case where dist.all.reduced < 1)
-              dist.all.reduced$dt = 0
-            }else{ #if there's more than one row in dist.all.reduced
-              timesFrame = data.frame(dist.all.reduced$dateTime[1:(nrow(dist.all.reduced) - 1)], dist.all.reduced$dateTime[2:nrow(dist.all.reduced)], stringsAsFactors = TRUE)
+            if(nrow(x.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because x.reduced is only created from blocks with observed contacts, there will never be a case where x.reduced < 1)
+              x.reduced$dt = 0
+            }else{ #if there's more than one row in x.reduced
+              timesFrame = data.frame(x.reduced$dateTime[1:(nrow(x.reduced) - 1)], x.reduced$dateTime[2:nrow(x.reduced)], stringsAsFactors = TRUE)
               timedif = apply(timesFrame, 1, timeDifference)
-              dist.all.reduced$dt = c(0, timedif) 
+              x.reduced$dt = c(0, timedif) 
             }
           }
-          comboFrame = data.frame(unique(dist.all.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
-          duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, dist.all.reduced)	
+          comboFrame = data.frame(unique(x.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
+          duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, x.reduced)	
           durationTable = data.frame(data.table::rbindlist(duration), stringsAsFactors = TRUE)
           if(nrow(durationTable) > 0){ #if there was at least one contact duration, block information is appended to the data frame. If there are no observations, durationTable becomes NULL
-            durationTable$block<- unique(dist.all$block)
-            durationTable$block.start<- unique(dist.all$block.start)
-            durationTable$block.end<- unique(dist.all$block.end)
-            durationTable$numBlocks<- unique(dist.all$numBlocks)
+            durationTable$block<- unique(x$block)
+            durationTable$block.start<- unique(x$block.start)
+            durationTable$block.end<- unique(x$block.end)
+            durationTable$numBlocks<- unique(x$numBlocks)
           }else{
             durationTable <- NULL
           }
@@ -346,74 +354,84 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
       
       thisEnvironment<-environment() #tag the environment of the parent function so that sub-functions may work within it. This is done so that we don't have to clone the list of data frames in the various sub functions over and over again.
       
+      #in case this wasn't already done, we order by date and second. Note that we must order it in this round-about way (using the date and daySecond vectors) to prevent ordering errors that sometimes occurs with dateTime data. It takes a bit longer (especially with larger data sets), but that's the price of accuracy
+      daySecondList = lubridate::hour(x$dateTime) * 3600 + lubridate::minute(x$dateTime) * 60 + lubridate::second(x$dateTime) #This calculates a day-second
+      lub.dates = lubridate::date(x$dateTime)
+      
       if(blocking == TRUE){
         
-        if(length(x$block) == 0 ){ #If there's no "block" column in dist.all output, then we need to define blocks here. Note here that, if individuals wanted to append block information to the dist.all file (for whatever reason), they may do so using the timeblock.append function. If users did this, there's no need to waste time remaking blocks here. 
-          
-          #the code below comes from the contact::timeBlock.append function. We just refrain from calling it here to prevent the inputs being cloned within the function, and thus save memory.
-          
-          if(blockUnit == "Secs" || blockUnit == "SECS" || blockUnit == "secs"){
-            blockLength1 <- blockLength
-          }
-          if(blockUnit == "Mins" || blockUnit == "MINS" || blockUnit == "mins"){
-            blockLength1 <- blockLength*60 #num seconds in a minute
-          }
-          if(blockUnit == "Hours" || blockUnit == "HOURS" || blockUnit == "hours"){
-            blockLength1 <- blockLength*60*60 #num seconds in an hour
-          }
-          if(blockUnit == "Days" || blockUnit == "DAYS" || blockUnit == "days"){
-            blockLength1 <- blockLength*60*60*24 #num seconds in a day
-          }
-          if(blockUnit == "Weeks" || blockUnit == "WEEKS" || blockUnit == "weeks"){
-            blockLength1 <- blockLength*60*60*24*7 #num seconds in a week
-          }
-          
-          daySecondList = lubridate::hour(x$dateTime) * 3600 + lubridate::minute(x$dateTime) * 60 + lubridate::second(x$dateTime) #This calculates a day-second
-          lub.dates = lubridate::date(x$dateTime)
-          x<-x[order(lub.dates, daySecondList),] #in case this wasn't already done, we order by date and second. Note that we must order it in this round-about way (using the date and daySecond vectors) to prevent ordering errors that sometimes occurs with dateTime data
-          
-          #for some odd reason, difftime will output mostly zeroes (incorrectly) if there are > 1 correct 0 at the beginning. We use a crude fix here to address this. Basically, we create the zeroes first and combine it with other values afterwards
-          totSecond <- rep(0, length(which(x$dateTime == x$dateTime[1])))
-          totSecond2<-as.integer(difftime(x$dateTime[(length(totSecond) +1): nrow(x)] ,x$dateTime[1] , units = c("secs")))
-          studySecond <- as.integer((c(totSecond, totSecond2) -min(c(totSecond, totSecond2))) + 1)
-          
-          numblocks <- ceiling((max(studySecond) - 1)/blockLength1)
-          block <-rep(0,length(studySecond))
-          for(g in 1:(numblocks -1)){ #numblocks - 1 because the last block in the dataset may be smaller than previous blocks (if blockLength1 does not divide evenly into timedif)
-            block[which(studySecond >= ((g-1)*blockLength1 + 1) & studySecond <= (g*blockLength1))] = g
-          }
-          if(length(which(block == 0)) > 0){ #identifies the last block
-            block[which(block == 0)] = numblocks
-          }
-          
-          block.start<-as.character(as.POSIXct(x$dateTime[1]) + ((block - 1)*blockLength1)) #identify the timepoint where each block starts (down to the second resolution)
-          block.end<-as.character(as.POSIXct(x$dateTime[1]) + ((block - 1)*blockLength1) + (blockLength1 -1)) #identify the timepoint where each block ends (down to the second resolution)
-          
-          x$block <- block
-          x$block.start <- block.start
-          x$block.end <- block.end
-          x$numBlocks <- max(block) #the contactTest function will require this information (i.e. the number of blocks in the dataset)
-          
-          rm(list = c("daySecondList", "lub.dates", "totSecond", "totSecond2", "studySecond", "block", "numblocks", "block.start", "block.end")) #remove these objects because they are no longer needed.
-          
-          blockList<-list()
-          blockVec <- unique(x$block)
-          
-          for(j in 1:length(blockVec)){
-            blockL1<- list(x[which(x$block == blockVec[j]),])
-            blockList<- c(blockList, blockL1)
-          }
-          
-        }else{ #if(length(x$block) > 0)
-          block = x$block
-          blockVec<-unique(block)
-          blockList<-list()
-          for(j in 1:length(blockVec)){
-            blockL1<- list(x[which(block == blockVec[j]),])
-            blockList<- c(blockList, blockL1)
-          }  
+        x<-x[order(lub.dates, daySecondList),] #in case this wasn't already done, we order by date and second. Note that we must order it in this round-about way (using the date and daySecond vectors) to prevent ordering errors that sometimes occurs with dateTime data
+        
+        #the code below comes from the contact::timeBlock.append function. We just refrain from calling it here to prevent the inputs being cloned within the function, and thus save memory.
+        
+        if(blockUnit == "Secs" || blockUnit == "SECS" || blockUnit == "secs"){
+          blockLength1 <- blockLength
+        }
+        if(blockUnit == "Mins" || blockUnit == "MINS" || blockUnit == "mins"){
+          blockLength1 <- blockLength*60 #num seconds in a minute
+        }
+        if(blockUnit == "Hours" || blockUnit == "HOURS" || blockUnit == "hours"){
+          blockLength1 <- blockLength*60*60 #num seconds in an hour
+        }
+        if(blockUnit == "Days" || blockUnit == "DAYS" || blockUnit == "days"){
+          blockLength1 <- blockLength*60*60*24 #num seconds in a day
+        }
+        if(blockUnit == "Weeks" || blockUnit == "WEEKS" || blockUnit == "weeks"){
+          blockLength1 <- blockLength*60*60*24*7 #num seconds in a week
         }
         
+        if(length(blockingStartTime) == 1){ #if the blockingStartTime argument is defined, we calculate how far it is away (in seconds) from the minimum timepoint in x
+          
+          blockTimeAdjustment <- difftime(x$dateTime[1], blockingStartTime, units = c("secs"))
+          
+        }else{ #if the blockingStartTime argument is NOT defined, the adjustment is 0
+          
+          blockTimeAdjustment <- 0
+          
+        }
+        
+        #for some odd reason, difftime will output mostly zeroes (incorrectly) if there are > 1 correct 0 at the beginning. We use a crude fix here to address this. Basically, we create the zeroes first and combine it with other values afterwards
+        totSecond <- rep(0, length(which(x$dateTime == x$dateTime[1])))
+        if(nrow(x) > length(totSecond)){
+          totSecond2<-as.integer(difftime(x$dateTime[(length(totSecond) +1): nrow(x)] ,x$dateTime[1], units = c("secs")))
+        }else{
+          totSecond2 <- NULL
+        }
+        studySecond <- as.integer((c(totSecond, totSecond2) -min(c(totSecond, totSecond2))) + 1) + blockTimeAdjustment
+        
+        numblocks <- as.integer(ceiling(max(studySecond)/blockLength1))
+        block<- ceiling(studySecond/blockLength1)
+        
+        #numblocks <- as.integer(ceiling((max(studySecond) - 1)/blockLength1))
+        #block <-rep(0,length(studySecond))
+        #for(g in 1:(numblocks -1)){ #numblocks - 1 because the last block in the dataset may be smaller than previous blocks (if blockLength1 does not divide evenly into timedif)
+        #  block[which(studySecond >= ((g-1)*blockLength1 + 1) & studySecond <= (g*blockLength1))] = g
+        #}
+        #if(length(which(block == 0)) > 0){ #identifies the last block
+        #  block[which(block == 0)] = numblocks
+        #}
+        
+        block.start<-as.character((as.POSIXct(x$dateTime[1]) - blockTimeAdjustment) + ((block - 1)*blockLength1)) #identify the timepoint where each block starts (down to the second resolution)
+        block.end<-as.character((as.POSIXct(x$dateTime[1]) - blockTimeAdjustment) + ((block - 1)*blockLength1) + (blockLength1 -1)) #identify the timepoint where each block ends (down to the second resolution)
+        
+        x$block <- block
+        x$block.start <- block.start
+        x$block.end <- block.end
+        x$numBlocks <- max(block) #the contactTest function will require this information (i.e. the number of blocks in the dataset)
+        
+        x<-x[order(x$block, x$id, studySecond),] #reorder x to ensure that block and individual ids are prioritized over dateTime information
+        
+        rm(list = c("totSecond", "totSecond2", "studySecond", "block", "numblocks", "block.start", "block.end")) #remove these objects because they are no longer needed.
+        
+        blockList<-list()
+        blockVec <- unique(x$block)
+        
+        for(j in 1:length(blockVec)){
+          blockL1<- list(droplevels(x[which(x$block == blockVec[j]),]))
+          blockList<- c(blockList, blockL1)
+        }
+        
+        rm(list = c("daySecondList", "lub.dates")) #remove these objects because they are no longer needed.
         rm(x) #remove x to free up local memory
         
         if(parallel == TRUE){#if parallel is true, we run blocks in parallel
@@ -437,6 +455,10 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
           durationTable[,match("numBlocks", names(durationTable))]<- as.factor(durationTable[,match("numBlocks", names(durationTable))])       
         }
       }else{ #If blocking == FALSE
+        
+        x<-x[order(x$id, lub.dates, daySecondList),] #order x (Note this is a bit different from how x should be ordered if blocking == TRUE)
+        rm(list = c("daySecondList", "lub.dates")) #remove these objects because they are no longer needed.
+        
         durationTable <- durFinder.noblock(parallel, dist.threshold, sec.threshold, equidistant.time, nCores, environmentTag = thisEnvironment) #note that if blocking == FALSE, we can still improve speed a bit by parallelizing a later sub function. Further note that x isn't carried over because the function works in the master-function environment
       }
       if(nrow(durationTable) > 0){ 
@@ -534,8 +556,8 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
           
           return(durationTab)
         }
-        contactMatrix.maker <- function(x,idVec1, dist.all.reduced){
-          spec.dist = dist.all.reduced[which(dist.all.reduced$id == as.character(unname(unlist(x[1])))),]
+        contactMatrix.maker <- function(x,idVec1, x.reduced){
+          spec.dist = x.reduced[which(x.reduced$id == as.character(unname(unlist(x[1])))),]
           dt = spec.dist$dt
           timebreakVec <-which(dt > unname(unlist(x[3]))) #needed here. fixed 1/8
           distmat = data.matrix(spec.dist[,c(match(paste("dist.to.indiv_",as.character(idVec1),sep =""), names(spec.dist)))])
@@ -549,36 +571,35 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
           return(idDurations)
         }
         
-        dist.all<-x[order(x$id, x$dateTime),]
-        idVec1 = unique(dist.all$id)
+        idVec1 = unique(x$id)
         if(length(idVec1) > 1){
-          dist.all.reduced <-dist.all[-which(dist.all$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
+          x.reduced <-x[-which(x$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
           
           if(equidistant.time == TRUE){ #added 02/04/2019 to make the dt calculations a toggleable parameter that users may turn off if all data points in their data set are temporally equidistant. This saves a large amount of time (approx. 3.5 mins/day)
-            dist.all.reduced$dt = 0
+            x.reduced$dt = 0
           }else{ #if equidistant.time == FALSE
             
-            if(nrow(dist.all.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because dist.all.reduced is only created from blocks with observed contacts, there will never be a case where dist.all.reduced < 1)
-              dist.all.reduced$dt = 0
-            }else{ #if there's more than one row in dist.all.reduced
-              timesFrame = data.frame(dist.all.reduced$dateTime[1:(nrow(dist.all.reduced) - 1)], dist.all.reduced$dateTime[2:nrow(dist.all.reduced)], stringsAsFactors = TRUE)
+            if(nrow(x.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because x.reduced is only created from blocks with observed contacts, there will never be a case where x.reduced < 1)
+              x.reduced$dt = 0
+            }else{ #if there's more than one row in x.reduced
+              timesFrame = data.frame(x.reduced$dateTime[1:(nrow(x.reduced) - 1)], x.reduced$dateTime[2:nrow(x.reduced)], stringsAsFactors = TRUE)
               if (parallel == TRUE){
                 cl<-parallel::makeCluster(nCores)
                 on.exit(parallel::stopCluster(cl))
                 timedif<-parallel::parApply(cl, timesFrame, 1, timeDifference)
-                dist.all.reduced$dt = c(0, timedif) #timedif represents the time it takes to move from location i-1 to location i	
+                x.reduced$dt = c(0, timedif) #timedif represents the time it takes to move from location i-1 to location i	
               }else{
                 timedif = apply(timesFrame, 1, timeDifference) 
-                dist.all.reduced$dt = c(0, timedif) 
+                x.reduced$dt = c(0, timedif) 
               }
             }
           }
-          comboFrame = data.frame(unique(dist.all.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
+          comboFrame = data.frame(unique(x.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
           
           if (parallel == TRUE){
-            duration<-parallel::parApply(cl, comboFrame, 1, contactMatrix.maker,idVec1, dist.all.reduced) #note that cl is defined above
+            duration<-parallel::parApply(cl, comboFrame, 1, contactMatrix.maker,idVec1, x.reduced) #note that cl is defined above
           }else{
-            duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, dist.all.reduced)	
+            duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, x.reduced)	
           }
           durationTable = data.frame(data.table::rbindlist(duration), stringsAsFactors = TRUE)
         }else{
@@ -651,8 +672,8 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
         
         return(durationTab)
       }
-      contactMatrix.maker <- function(x,idVec1, dist.all.reduced){
-        spec.dist = dist.all.reduced[which(dist.all.reduced$id == as.character(unname(unlist(x[1])))),]
+      contactMatrix.maker <- function(x,idVec1, x.reduced){
+        spec.dist = x.reduced[which(x.reduced$id == as.character(unname(unlist(x[1])))),]
         dt = spec.dist$dt
         timebreakVec <-which(dt > unname(unlist(x[3]))) #needed here. fixed 1/8
         distmat = data.matrix(spec.dist[,c(match(paste("dist.to.indiv_",as.character(idVec1),sep =""), names(spec.dist)))])
@@ -666,30 +687,29 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
         return(idDurations)
       }   
       
-      dist.all<-x[order(x$id, x$dateTime),]
-      idVec1 = unique(dist.all$id)
+      idVec1 = unique(x$id)
       if(length(idVec1) > 1){
-        dist.all.reduced <-dist.all[-which(dist.all$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
+        x.reduced <-x[-which(x$id == idVec1[length(idVec1)]),] #there's no need to process contacts associated with the last id values, because if they contacted any other individuals, the contacts would already be processed earlier on.
         
         if(equidistant.time == TRUE){ #added 02/04/2019 to make the dt calculations a toggleable parameter that users may turn off if all data points in their data set are temporally equidistant. This saves a large amount of time (approx. 3.5 mins/day)
-          dist.all.reduced$dt = 0
+          x.reduced$dt = 0
         }else{ #if equidistant.time == FALSE
-          if(nrow(dist.all.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because dist.all.reduced is only created from blocks with observed contacts, there will never be a case where dist.all.reduced < 1)
-            dist.all.reduced$dt = 0
-          }else{ #if there's more than one row in dist.all.reduced
-            timesFrame = data.frame(dist.all.reduced$dateTime[1:(nrow(dist.all.reduced) - 1)], dist.all.reduced$dateTime[2:nrow(dist.all.reduced)], stringsAsFactors = TRUE)
+          if(nrow(x.reduced) ==1){ #if there's only one row, there cannot be any time difference (note, because x.reduced is only created from blocks with observed contacts, there will never be a case where x.reduced < 1)
+            x.reduced$dt = 0
+          }else{ #if there's more than one row in x.reduced
+            timesFrame = data.frame(x.reduced$dateTime[1:(nrow(x.reduced) - 1)], x.reduced$dateTime[2:nrow(x.reduced)], stringsAsFactors = TRUE)
             timedif = apply(timesFrame, 1, timeDifference)
-            dist.all.reduced$dt = c(0, timedif) 
+            x.reduced$dt = c(0, timedif) 
           }
         }
-        comboFrame = data.frame(unique(dist.all.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
-        duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, dist.all.reduced)	
+        comboFrame = data.frame(unique(x.reduced$id),dist.threshold,sec.threshold, stringsAsFactors = TRUE)
+        duration = apply(comboFrame, 1, contactMatrix.maker,idVec1, x.reduced)	
         durationTable = data.frame(data.table::rbindlist(duration), stringsAsFactors = TRUE)
         if(nrow(durationTable) > 0){ #if there was at least one contact duration, block information is appended to the data frame. If there are no observations, durationTable becomes NULL
-          durationTable$block<- unique(dist.all$block)
-          durationTable$block.start<- unique(dist.all$block.start)
-          durationTable$block.end<- unique(dist.all$block.end)
-          durationTable$numBlocks<- unique(dist.all$numBlocks)
+          durationTable$block<- unique(x$block)
+          durationTable$block.start<- unique(x$block.start)
+          durationTable$block.end<- unique(x$block.end)
+          durationTable$numBlocks<- unique(x$numBlocks)
         }else{
           durationTable <- NULL
         }
@@ -701,10 +721,14 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
     
     thisEnvironment<-environment() #tag the environment of the parent function so that sub-functions may work within it. This is done so that we don't have to clone the list of data frames in the various sub functions over and over again.
     
+    #in case this wasn't already done, we order by date and second. Note that we must order it in this round-about way (using the date and daySecond vectors) to prevent ordering errors that sometimes occurs with dateTime data. It takes a bit longer (especially with larger data sets), but that's the price of accuracy
+    daySecondList = lubridate::hour(x$dateTime) * 3600 + lubridate::minute(x$dateTime) * 60 + lubridate::second(x$dateTime) #This calculates a day-second
+    lub.dates = lubridate::date(x$dateTime)
+    
     if(blocking == TRUE){
       
-      if(length(x$block) == 0 ){ #If there's no "block" column in dist.all output, then we need to define blocks here. Note here that, if individuals wanted to append block information to the dist.all file (for whatever reason), they may do so using the timeblock.append function. If users did this, there's no need to waste time remaking blocks here. 
-        
+      x<-x[order(lub.dates, daySecondList),] #in case this wasn't already done, we order by date and second. Note that we must order it in this round-about way (using the date and daySecond vectors) to prevent ordering errors that sometimes occurs with dateTime data
+      
         #the code below comes from the contact::timeBlock.append function. We just refrain from calling it here to prevent the inputs being cloned within the function, and thus save memory.
         
         if(blockUnit == "Secs" || blockUnit == "SECS" || blockUnit == "secs"){
@@ -723,16 +747,26 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
           blockLength1 <- blockLength*60*60*24*7 #num seconds in a week
         }
         
-        daySecondList = lubridate::hour(x$dateTime) * 3600 + lubridate::minute(x$dateTime) * 60 + lubridate::second(x$dateTime) #This calculates a day-second
-        lub.dates = lubridate::date(x$dateTime)
-        x<-x[order(lub.dates, daySecondList),] #in case this wasn't already done, we order by date and second. Note that we must order it in this round-about way (using the date and daySecond vectors) to prevent ordering errors that sometimes occurs with dateTime data
+        if(length(blockingStartTime) == 1){ #if the blockingStartTime argument is defined, we calculate how far it is away (in seconds) from the minimum timepoint in x
+          
+          blockTimeAdjustment <- difftime(x$dateTime[1], blockingStartTime, units = c("secs"))
+          
+        }else{ #if the blockingStartTime argument is NOT defined, the adjustment is 0
+          
+          blockTimeAdjustment <- 0
+          
+        }
         
         #for some odd reason, difftime will output mostly zeroes (incorrectly) if there are > 1 correct 0 at the beginning. We use a crude fix here to address this. Basically, we create the zeroes first and combine it with other values afterwards
         totSecond <- rep(0, length(which(x$dateTime == x$dateTime[1])))
-        totSecond2<-as.integer(difftime(x$dateTime[(length(totSecond) +1): nrow(x)] ,x$dateTime[1] , units = c("secs")))
-        studySecond <- as.integer((c(totSecond, totSecond2) -min(c(totSecond, totSecond2))) + 1)
+        if(nrow(x) > length(totSecond)){
+          totSecond2<-as.integer(difftime(x$dateTime[(length(totSecond) +1): nrow(x)] ,x$dateTime[1], units = c("secs")))
+        }else{
+          totSecond2 <- NULL
+        }
+        studySecond <- as.integer((c(totSecond, totSecond2) -min(c(totSecond, totSecond2))) + 1) + blockTimeAdjustment
         
-        numblocks <- ceiling((max(studySecond) - 1)/blockLength1)
+        numblocks <- as.integer(ceiling((max(studySecond) - 1)/blockLength1))
         block <-rep(0,length(studySecond))
         for(g in 1:(numblocks -1)){ #numblocks - 1 because the last block in the dataset may be smaller than previous blocks (if blockLength1 does not divide evenly into timedif)
           block[which(studySecond >= ((g-1)*blockLength1 + 1) & studySecond <= (g*blockLength1))] = g
@@ -741,34 +775,27 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
           block[which(block == 0)] = numblocks
         }
         
-        block.start<-as.character(as.POSIXct(x$dateTime[1]) + ((block - 1)*blockLength1)) #identify the timepoint where each block starts (down to the second resolution)
-        block.end<-as.character(as.POSIXct(x$dateTime[1]) + ((block - 1)*blockLength1) + (blockLength1 -1)) #identify the timepoint where each block ends (down to the second resolution)
+        block.start<-as.character((as.POSIXct(x$dateTime[1]) - blockTimeAdjustment) + ((block - 1)*blockLength1)) #identify the timepoint where each block starts (down to the second resolution)
+        block.end<-as.character((as.POSIXct(x$dateTime[1]) - blockTimeAdjustment) + ((block - 1)*blockLength1) + (blockLength1 -1)) #identify the timepoint where each block ends (down to the second resolution)
         
         x$block <- block
         x$block.start <- block.start
         x$block.end <- block.end
         x$numBlocks <- max(block) #the contactTest function will require this information (i.e. the number of blocks in the dataset)
         
-        rm(list = c("daySecondList", "lub.dates", "totSecond", "totSecond2", "studySecond", "block", "numblocks", "block.start", "block.end")) #remove these objects because they are no longer needed.
+        x<-x[order(x$block, x$id, studySecond),] #reorder x to ensure that block and individual ids are prioritized over dateTime information
+        
+        rm(list = c("totSecond", "totSecond2", "studySecond", "block", "numblocks", "block.start", "block.end")) #remove these objects because they are no longer needed.
         
         blockList<-list()
         blockVec <- unique(x$block)
         
         for(j in 1:length(blockVec)){
-          blockL1<- list(x[which(x$block == blockVec[j]),])
+          blockL1<- list(droplevels(x[which(x$block == blockVec[j]),]))
           blockList<- c(blockList, blockL1)
         }
-        
-      }else{ #if(length(x$block) > 0)
-        block = x$block
-        blockVec<-unique(block)
-        blockList<-list()
-        for(j in 1:length(blockVec)){
-          blockL1<- list(x[which(block == blockVec[j]),])
-          blockList<- c(blockList, blockL1)
-        }  
-      }
       
+      rm(list = c("daySecondList", "lub.dates")) #remove these objects because they are no longer needed.
       rm(x) #remove x to free up local memory
       
       if(parallel == TRUE){#if parallel is true, we run blocks in parallel
@@ -792,6 +819,10 @@ contactDur.all<-function(x,dist.threshold=1,sec.threshold=10, blocking = FALSE,
         durationTable[,match("numBlocks", names(durationTable))]<- as.factor(durationTable[,match("numBlocks", names(durationTable))])       
       }
     }else{ #If blocking == FALSE
+      
+      x<-x[order(x$id, lub.dates, daySecondList),] #order x (Note this is a bit different from how x should be ordered if blocking == TRUE)
+      rm(list = c("daySecondList", "lub.dates")) #remove these objects because they are no longer needed.
+      
       durationTable <- durFinder.noblock(parallel, dist.threshold, sec.threshold, equidistant.time, nCores, environmentTag = thisEnvironment) #note that if blocking == FALSE, we can still improve speed a bit by parallelizing a later sub function. Further note that x isn't carried over because the function works in the master-function environment
     }
     if(nrow(durationTable) > 0){ 
